@@ -9,7 +9,7 @@ plot.map = TRUE,
 conf = 0.95, 
 opt = c("none", "inter", "slope"),
 opt.step = c(50, 50), 
-stop.crit = c("midpoint", "outlier"), 
+stop.crit = c("outlier", "midpoint"), 
 quan = 0.5, 
 slope = NULL, 
 count = 1)
