@@ -21,9 +21,8 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
     if (!is.numeric(which.cp)) which.cp <- match.arg(which.cp)
     type.eff <- match.arg(type.eff)
     
-    ## added option of external efficiencies or threshold cycles,
-    ## either single value (recycled) or a vector of values
-    ## in version 1.3-4.
+    ## from 1.3-4: added option of external efficiencies or threshold cycles,
+    ## either single value (recycled) or a vector of values.
     ## Check if all efficiencies are in [1, 2]
     ## if all is o.k., add to 'pcrbatch' data
     if (is.numeric(which.eff)) {
@@ -37,6 +36,7 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
       data <- rbind(data, effDAT) 
       which.eff <- "ext"     
     }
+      
     if (is.numeric(which.cp)) {
       if (length(which.cp) != ncol(data) - 1 ) stop("Length of input threshold cycles does not match number of runs!")
       cpDAT <- matrix(c("sig.ext", which.cp), nrow = 1)
@@ -45,16 +45,26 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
       which.cp <- "ext"      
     }      
     
-    ## added mak3 parameter in version 1.3-4
+    ## from 1.3-4: added mak3 parameter
     if (which.eff == "mak") {
       if (length(grep("mak3", data[, 1])) == 0) stop("'data' has no mak3 model included! Please use 'pcrbatch' with 'do.mak = TRUE'!")
       WHICH <- which(data[, 1] == "mak3.D0")
       data[, 1] <- sub("mak3.D0", "mak.eff", data[, 1])
     }    
     
-    DATA <- data[, -1]         
+    DATA <- data[, -1]
+
+    ## added removal of failed runs (either failed fits
+    ## or SOD outlier) from DATA and 'group' by identification
+    ## of *...* or **...** in sample name in version 1.3-5
+    sampNAMES <- names(DATA)
+    hasTag <- grep("\\*[[:print:]]*\\*", sampNAMES)
+    if (length(hasTag) > 0) {
+       DATA <- DATA[, -hasTag]
+       group <- group[-hasTag]
+    }
+    
     cpNAMES <- effNAMES <- NULL
-            
     PATTERN <- unique(group)
     
     ## test for presence of reference genes
@@ -63,18 +73,18 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
     ## check for replicate data, if not present set type.eff = "individual"
     REPS <- lapply(PATTERN, function(x) which(x == group))
     NREPS <- sapply(REPS, function(x) length(x))           
-    if (!all(NREPS > 1)) type.eff <- "individual"    
+    if (!all(NREPS > 1)) type.eff <- "individual"
     
-    ## initialize data as time-series 
+    ## initialize data as time-series
     cpDAT <- effDAT <- ts()     
     
-    ## for all entries 'gs', 'gc', 'rs', 'rc' do...     
+    ## for all entries 'gs', 'gc', 'rs', 'rc' do...
     for (i in 1:length(PATTERN)) {
       WHICH <- which(group == PATTERN[i]) 
-      cpSEL <- which(data[, 1] == paste("sig.", which.cp, sep = ""))          
-      effSEL <- which(data[, 1] == paste(which.eff, ".eff", sep = ""))  
+      cpSEL <- which(data[, 1] == paste("sig.", which.cp, sep = ""))
+      effSEL <- which(data[, 1] == paste(which.eff, ".eff", sep = ""))
       if (length(WHICH) != 1) tempCP <- as.numeric(t(DATA[cpSEL, WHICH])) else tempCP <- as.numeric(as.vector(DATA[cpSEL, WHICH]))     
-      if (length(WHICH) != 1) tempEff <- as.numeric(t(DATA[effSEL, WHICH])) else tempEff <- as.numeric(as.vector(DATA[effSEL, WHICH]))       
+      if (length(WHICH) != 1) tempEff <- as.numeric(t(DATA[effSEL, WHICH])) else tempEff <- as.numeric(as.vector(DATA[effSEL, WHICH]))
       if (is.numeric(which.eff)) tempEff <- rep(which.eff, length(WHICH))
       cpDAT <- cbind(cpDAT, ts(tempCP))            
       effDAT <- cbind(effDAT, ts(tempEff))
@@ -111,6 +121,7 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
     if (refNo) {
       EXPR <- expression(eff.gc^cp.gc/eff.gs^cp.gs)
       TIES <- c(1, 2, 1, 2)  
+      
       ## added mak3 option (1.3-4) => we only need D0 for ratio calculation
       if (which.eff == "mak") {
         EXPR <- expression(eff.gs/eff.gc)
@@ -120,6 +131,7 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
     else {
       EXPR <- expression((eff.gc^cp.gc/eff.gs^cp.gs)/(eff.rc^cp.rc/eff.rs^cp.rs))
       TIES <- c(1, 2, 1, 2, 1, 2, 1, 2)
+      
       ## added mak3 option (1.3-4) => we only need D0 for ratio calculation
       if (which.eff == "mak") {
         EXPR <- expression((eff.gs/eff.gc)/(eff.rs/eff.rc))
@@ -127,9 +139,11 @@ which.cp = c("cpD2", "cpD1", "cpE", "cpR", "cpT", "Cy0"),
       }      
     }       
             
-    CRIT <- c("perm > init", "perm == init", "perm < init")             
+    CRIT <- c("perm > init", "perm == init", "perm < init")
+              
     PROP <- try(propagate(EXPR, allDAT, do.sim = TRUE, do.perm = TRUE, ties = TIES, perm.crit = CRIT, 
                       verbose = TRUE, logx = TRUE, ...))
+
     if (inherits(PROP, "try-error")) stop("'propagate' failed to calculate ratios! Try other 'which.eff', 'type.eff' or 'which.cp'.")
     PROP <- c(list(data = allDAT), PROP)     
     class(PROP) <- "ratiocalc"
